/*
 * GXSNMP -- An snmp mangament application
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>   
#include <getopt.h>
#include <strings.h>
#include <signal.h>
#include <errno.h>
#include <malloc.h>
#include <sys/time.h>
#include <sys/socket.h>
#include "g_snmp.h"

#define I_AM_REALLY_BRAVE

int end;
int retry, timeout, version;
char *name, *variable;
char result[1024];

extern int snmpsocket;

void
cb_time(host_snmp *host, void *magic)
{
  end = 1;
  strcpy(result, "Query timed out");
}

void
cb_done (host_snmp *host, void *magic, SNMP_PDU *spdu, GSList *objs)
{
  end = 1;
  if (objs)
    {
      g_snmp_printf(result, sizeof(result), objs->data);
    }
}

static
snmp_get()
{
  struct timeval to;
  int numfds;
  fd_set fdset;
  int count;
  host_snmp *host;
  GSList *pdu = NULL;

#ifdef I_AM_REALLY_BRAVE
  GString *epdu;
  GString *community;
  ASN1_SCK asn1;
  SNMP_PDU pduh;

  struct servent *port;
  struct hostent *hp;
  struct sockaddr_in address;

  char *buffer;
  char *ptr;
  guint blen;

  g_snmp_init();
  snmpsocket = socket(AF_INET, SOCK_DGRAM, 0);

  address.sin_family = AF_INET;
  hp = gethostbyname(name);
  if (!hp)
    return;

  g_memmove((char *)&address.sin_addr, (char *)hp->h_addr, hp->h_length);
  port = getservbyname("snmp","udp");
  if (!port)
    return;
  address.sin_port = port->s_port;

  community = g_string_new("public");
  g_pdu_add_name(&pdu, variable, SNMP_NULL, NULL);

  buffer = g_malloc(65536);
  ptr = buffer;
  blen = 65536;

  pduh.request.id           = 1;
  pduh.request.type         = SNMP_PDU_GET;
  pduh.request.error_status = 0;
  pduh.request.error_index  = 0;

  g_asn1_open (&asn1, ptr, blen, ASN1_ENC);

  g_snmp_pdu_v1_encode (&asn1, &pduh, pdu);

  g_asn1_close (&asn1, &ptr, &blen);
  epdu->len = blen;
  epdu->str = ptr;

  count=sendPdu(1, &address, PMODEL_SNMPV1, SMODEL_SNMPV1, community, SLEVEL_NANP, 
        NULL, NULL, 1, epdu, FALSE);

  free(buffer);

#else
  host = malloc(sizeof(host_snmp));

  host->rcomm      = "public";
  host->wcomm      = "public";
  host->retries    = retry;
  host->timeout    = timeout;
  host->timeoutval = timeout;
  host->version    = version;
  host->name       = name;
  host->port       = 0;
  host->status     = 0;
  host->magic      = NULL;
  host->done_callback = cb_done;
  host->time_callback = cb_time;

  snmpsocket = socket(AF_INET, SOCK_DGRAM, 0);
  g_pdu_add_name(&pdu, variable, SNMP_NULL, NULL);
  g_async_get (host, pdu);

  end = 0;
  while (!end) 
    {
      to.tv_sec = 1;
      to.tv_usec = 0;

      numfds = snmpsocket+1;
      FD_ZERO(&fdset);
      FD_SET(snmpsocket, &fdset);
      count = select(numfds, &fdset, 0, 0, &to);
      if (count > 0)  
        g_snmp_input_cb(NULL);
      else
        g_snmp_timeout_cb(NULL);
    }
#endif
}

static int
usage()
{
  printf("\n"
         "Usage: test [-t timeout] [-r retry] [-v version] host variable\n"
         "\n"
         "-t timeout:   time to wait for a SNMP response\n"
         "-r retry:     number of retries until a node is considered unreachable.\n"
         "-h            print this message.\n\n");
  exit(1);
}

int main(int argc, char *argv[])
{
  int c;
  int oi = 0;


  static const struct option long_options[] = 
    {
      {"retry",    required_argument, 0, 'r'},
      {"timeout",  required_argument, 0, 't'},
      {"version",  required_argument, 0, 'v'},
      {"engineid", required_argument, 0, 'e'},
      {"secname",  required_argument, 0, 'c'},
      {"secmodel", required_argument, 0, 's'},
      {"seclevel", required_argument, 0, 'l'},
      {"debug",    no_argument,       0, 'x'},
      {"help",     no_argument,       0, 'h'},
      {0, 0, 0, 0}
    };

  while (1) 
    {
      c = getopt_long(argc, argv, "c:o:p:m:t:r:d:xh?", long_options, &oi);

      if (c == -1)
        break;

      switch(c) 
        {
          case 'r':
            retry = atoi(optarg);
            break;
          case 't':
            timeout = atoi(optarg);
            break;
          case 'v':
            version = atoi(optarg);
            break;
          case 'h':
            usage();
            break;
        }
    }
  if (argc - optind < 2) 
    usage();

  name     = argv[optind];
  variable = argv[optind+1];

  init_mib();
  printf("sending SNMPGET to %s for %s\n", name, variable);
  snmp_get();

  printf("%s\n", result);
}
